import java.io.*;
import java.util.*;

public class TestGen {

    public static void main(String[] args) throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        desc = new PrintWriter("../tests.desc");

        done = 0;
        gen_samples();

        //-------------------------first group-------------------------------------------------
        subtask = "subtask1";

        genRand(5, 5);
        genRand(1, 5);
        genRand(5, 1);
        genOneSwap1(10, 10);
        genOneSwap2(4, 17);
        genOneSwap3(20, 4);
        genSpecial(7, 7);
        genLOL(21, 5);

        for (int i = 0; i < 3; i++) {
            genOneSwap1(nand(MAXN1), nand(MAXN1));
            genOneSwap2(nand(MAXN1), nand(MAXN1));
            genOneSwap3(nand(MAXN1), nand(MAXN1));
            genSpecial(nand(MAXN1), nand(MAXN1));
            genLOL(nand(MAXN1), nand(MAXN1));
        }

        for (int i = 0; i < 15; i++) {
            genRand(nand(MAXN1), nand(MAXN1));
        }

        genRand(MAXN1, MAXN1);
        genOneSwap1(MAXN1, MAXN1);
        genOneSwap2(MAXN1, MAXN1);
        genOneSwap3(MAXN1, MAXN1);
        genSpecial(MAXN1, MAXN1);
        genLOL(MAXN1, MAXN1);


        desc.println(String.format("comment_subtask%d=\"n, m &lt;= %d\"", 1, MAXN1, 0));
        desc.println(String.format("bonus_subtask%d=%d", 1, 50));

        //------------------------second group------------------------------------------------
        subtask = "subtask2";

        for (int i = 0; i < 3; i++) {
            genOneSwap1(nand(MAXN1), nand(MAXN2));
            genOneSwap2(nand(MAXN2), nand(MAXN2));
            genOneSwap3(nand(MAXN2), nand(MAXN2));
            genSpecial(nand(MAXN2), nand(MAXN2));
            genLOL(nand(MAXN2), nand(MAXN2));
        }

        for (int i = 0; i < 15; i++) {
            genRand(nand(MAXN2), nand(MAXN2));
        }

        genRand(MAXN2, MAXN2);
        genOneSwap1(MAXN2, MAXN2);
        genOneSwap2(MAXN2, MAXN2);
        genOneSwap3(MAXN2, MAXN2);
        genSpecial(MAXN2, MAXN2);
        genLOL(MAXN2, MAXN2);

        desc.println(String.format("comment_subtask%d=\"n, m &lt;= %d\"", 2, MAXN2));
        desc.println(String.format("bonus_subtask%d=%d", 2, 50));


        desc.close();
    }

    static int done = 0;
    static PrintWriter desc;
    static final Random rnd = new Random(3289214075201L);
    static final int MAXN1 = 1000;
    static final int MAXN2 = 100_000;

    static int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    static int nand(int n) {
        return rand(n / 2) + n / 2;
    }

    static boolean trueOrFalse() {
        return rnd.nextBoolean();
    }

    static class Pair {
        int i, j;
        Pair(){}
        Pair(int i, int j) {
            this.i = i;
            this.j = j;
        }
        void swap() {
            int tmp = i;
            i = j;
            j = tmp;
        }
    }

    static void print(int n, Pair[] a) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out;
        if (done < 10)
            out = new PrintWriter("../tests/0" + String.valueOf(done));
        else
            out = new PrintWriter("../tests/" + String.valueOf(done));
        out.println(String.format("%d", n));
        out.println(String.format("%d", a.length));
        for (int i = 0; i < a.length; i++) {
            out.println(String.format("%d %d", a[i].i, a[i].j));
        }
        out.close();
    }

    static String getPath() {
        if (done < 10)
            return "../tests/0" + String.valueOf(done);
        return "../tests/" + String.valueOf(done);
    }

    static void print_desc(String path, String folder, String subtask, int points, String comment) throws IOException {
        desc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, points, comment));
    }

    static String subtask;

    static void print_desc_t(String comment) throws IOException {
        print_desc(getPath(), "tests", subtask, 0, comment);
    }

    //begin generators

    static void gen_samples() throws IOException {
        PrintWriter out;
        String path;
        path = "../preliminary/01";
        out = new PrintWriter(path);
        out.println("3");
        out.println("3");
        out.println("1 4");
        out.println("2 5");
        out.println("3 6");
        out.close();
        print_desc(path, "preliminary", "subtask0", 0, "sample 1");
        path = "../preliminary/02";
        out = new PrintWriter(path);
        out.println("4");
        out.println("3");
        out.println("8 7");
        out.println("1 5");
        out.println("1 2");
        out.close();
        print_desc(path, "preliminary", "subtask0", 0, "sample 2");
    }

    static void genRand(int n, int m) throws IOException {
        Pair[] a = new Pair[m];
        for (int i = 0; i < m; i++) {
            a[i] = new Pair();
            a[i].i = rand(2 * n) + 1;
            a[i].j = rand(2 * n) + 1;
        }
        print(n, a);
        print_desc_t(String.format("random test, n = %d, m = %d", n, m));
    }

    static void genOneSwap1(int n, int m) throws IOException {
        int x = rand(n) + 1;
        int y = rand(n) + n + 1;
        Pair[] a = new Pair[m];
        for (int i = 0; i < m; i++) {
            a[i] = new Pair();
            a[i].i = x;
            a[i].j = y;
            if (trueOrFalse())
                a[i].swap();
        }
        print(n, a);
        print_desc_t(String.format("one swap on different sides, n = %d, m = %d", n, m));
    }

    static void genOneSwap2(int n, int m) throws IOException {
        int x = rand(n) + 1;
        int y = rand(n) + 1;
        Pair[] a = new Pair[m];
        for (int i = 0; i < m; i++) {
            a[i] = new Pair();
            a[i].i = x;
            a[i].j = y;
            if (trueOrFalse())
                a[i].swap();
        }
        print(n, a);
        print_desc_t(String.format("one swap on left side, n = %d, m = %d", n, m));
    }

    static void genOneSwap3(int n, int m) throws IOException {
        int x = rand(n) + n + 1;
        int y = rand(n) + n + 1;
        Pair[] a = new Pair[m];
        for (int i = 0; i < m; i++) {
            a[i] = new Pair();
            a[i].i = x;
            a[i].j = y;
            if (trueOrFalse())
                a[i].swap();
        }
        print(n, a);
        print_desc_t(String.format("one swap on right side, n = %d, m = %d", n, m));
    }

    static void genLOL(int n, int m) throws IOException {
        int x = rand(2 * n) + 1;
        Pair[] a = new Pair[m];
        for (int i = 0; i < m; i++) {
            a[i] = new Pair();
            a[i].i = x;
            a[i].j = x;
        }
        print(n, a);
        print_desc_t(String.format("LOL test i = j for all swaps, n = %d, m = %d", n, m));
    }

    static void genSpecial(int n, int m) throws IOException {
        int x = rand(2 * n) + 1;
        Pair[] a = new Pair[m];
        for (int i = 0; i < m; i++) {
            a[i] = new Pair();
            a[i].i = x;
            a[i].j = rand(2 * n) + 1;
            if (trueOrFalse())
                a[i].swap();
        }
        print(n, a);
        print_desc_t(String.format("one arg for swap constant, n = %d, m = %d", n, m));
    }


    //end generator

}
