import java.io.*;
import java.math.*;
import java.util.*;

public class TestGen {
    int testCount, subtaskCount;
    PrintWriter testsDesc;

    Random random = new Random(787788);
    int MAX_NK = 100_000;
    int MX = 1_000_000_000;
    int MOD = 1_000_000_007;
    
    int[][] samples1 = new int[][]{{1, 0, 1, 0, 1, 0, 1}};
    int[][][] samples2 = new int[][][]{{{1, 1}, {0, 0}, {1, 1}}};

    void write(int[] a, int[][] b, String folder, String subtask, int points, String comment) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);
        out.println(a.length + " " + b.length);
        for (int i = 0; i < a.length; i++) {
            out.print(a[i]);
            if (i + 1 == a.length) {
                out.println();
            } else {
                out.print(" ");
            }
        }
        for (int i = 0; i < b.length; i++) {
            out.print(b[i][0] + " " + b[i][1]);
            out.println();
        }
        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=%s", path, folder, subtask, points, comment));
        System.out.println(String.format("Test %d is generated", testCount));
    }


    void genPreliminary() throws IOException {
        for (int i = 0; i < samples1.length; i++) {
            write(samples1[i], samples2[i], "preliminary", "subtask0", 0, String.format("\"sample %d\"", i + 1));
        }
        testCount = 0;
    }

    void genRandom(int n, int k, int MX) throws IOException {
        int a[] = new int[n];
        int b[][] = new int[k][2];
        
        for (int i = 0; i < n; i++) {
            a[i] = random.nextInt(MX);
        }
        for (int i = 0; i < k; i++) {
            while (true) {
                b[i][0] = random.nextInt(MX);
                b[i][1] = random.nextInt(MX);
                if (b[i][0] > b[i][1]) {
                    int o = b[i][0];
                    b[i][0] = b[i][1];
                    b[i][1] = o;
                }
                if (b[i][1] - b[i][0] >= MX / Math.min(k + 1, 8)) {
                    break;
                }
            }
        }
        
        write(a, b, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"random, n=%d, k=%d\"", n, k));
    }             

    void genMax(int n, int k, int MX) throws IOException {
        int a[] = new int[n];
        int b[][] = new int[k][2];
        
        for (int i = 0; i < n; i++) {
            a[i] = MX;
        }
        for (int i = 0; i < k; i++) {
            b[i][0] = 0;
            b[i][1] = MX;
        }
        
        write(a, b, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"random, n=%d, k=%d\"", n, k));
    }             

    void genSubtask(int points) throws IOException {
        ++subtaskCount;

        if (subtaskCount == 1) {
            genRandom(10, 3, 16);
            genRandom(15, 2, 32);
            genRandom(20, 4, 64);
            for (int k = 1; k <= 4; k++) {
                genRandom(random.nextInt(90) + 11, k, 1 << 10);
            }
            for (int k = 1; k <= 4; k++) {
                genRandom(100, k, 1 << 20);
            }
            testsDesc.println(String.format("comment_subtask%d=\"n &lt; 100, k &lt; 4\"", subtaskCount));
        }

        if (subtaskCount == 2) {
            for (int i = 0; i < 10; i++) {
                genRandom(500, random.nextInt(10) * 10 + random.nextInt(10) + 1, MX);
            }
            genRandom(500, 10, MX);
            genRandom(500, 100, MX);
            genRandom(500, 100, MX);
            genMax(500, 100, MX);
            testsDesc.println(String.format("comment_subtask%d=\"n &lt; 500, k &lt; 100\"", subtaskCount));
        }

        if (subtaskCount == 3) {
            for (int k = 1; k <= 10; k++) {
                genRandom(100_000 / k, k, MX);
                genMax(100_000 / k, k, MX);
            }
            genRandom(100_000 / 50, 50, MX);
            genRandom(100_000 / 50, 50, MX);
            genRandom(100_000 / 50, 50, MX);
            genRandom(100_000 / 300, 300, MX);
            genMax(100_000 / 300, 300, MX);
            testsDesc.println(String.format("comment_subtask%d=\"nk &lt; 100000, k &lt; n\"", subtaskCount));
        }

        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));       
    }

    void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genPreliminary();
        genSubtask(20);
        genSubtask(40);
        genSubtask(40);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
