#include <sstream>
#include <vector>
#include <algorithm>
#include <cstring>
#include <cstdlib>
#include <iostream>
#include <string>
#include <cassert>
#include <ctime>
#include <map>
#include <math.h>
#include <cstdio>
#include <set>
#include <deque>
#include <memory.h>
#include <queue>

#pragma comment(linker, "/STACK:50000000")
typedef long long ll;

using namespace std; 

const int inf = 1e9;
const int mod = 1e9 + 7;
const int lg = 30;

int n, k;
vector<int> a, l, r;
vector<int> toXor, go[2];
vector<int> sum;

void ADD(int &x, int y) {
	x += y;
	if (x >= mod) x -= mod;
}

void addNew() {
	toXor.push_back(0);
	go[0].push_back(-1);
	go[1].push_back(-1);
	sum.push_back(0);
}

void init() {
	toXor.clear();
	go[0].clear();
	go[1].clear();
	sum.clear();
	addNew();
}

void push(int cur, int len) {
	if (toXor[cur] != 0) {
		for (int k = 0; k < 2; k++) if (go[k][cur] == -1) {
			go[k][cur] = toXor.size();
			addNew();
		}
		if ((toXor[cur] >> (lg - len - 1)) & 1) {
			toXor[cur] ^= 1 << (lg - len - 1);
			swap(go[0][cur], go[1][cur]);
		}
		for (int k = 0; k < 2; k++) toXor[go[k][cur]] ^= toXor[cur];
		toXor[cur] = 0;
	}
}

void upd(int v) {
	sum[v] = 0;
	if (go[0][v] != -1) ADD(sum[v], sum[go[0][v]]);
	if (go[1][v] != -1) ADD(sum[v], sum[go[1][v]]);
}

void add(int x, int y) {
	int cur = 0;
	for (int i = 0; i < lg; i++) {
		push(cur, i);
		int o = (x >> (lg - i - 1)) & 1;
		if (go[o][cur] == -1) {
			go[o][cur] = toXor.size();
			addNew();
		}
		ADD(sum[cur], y);
		cur = go[o][cur];
	}
	ADD(sum[cur], y);
}

int getSum(int v, int tl, int tr, int l, int r, int len) {
	if (l > r || v == -1) return 0;
	push(v, len);
	if (l == tl && r == tr) {
		return sum[v];
	}
	int tm = (tl + tr) >> 1;
	int a = getSum(go[0][v], tl, tm, l, min(r, tm), len + 1);
	int b = getSum(go[1][v], tm + 1, tr, max(l, tm + 1), r, len + 1);
	a += b;
	if (a >= mod) a -= mod;
	return a;
}

int fastsolve() {
	vector<int> dp(n);
	for (int j = 0; j < k; j++) {
		init();
		add(0, !j);
		for (int i = 0; i < n; i++) {
			toXor[0] ^= a[i];
			int vv = getSum(0, 0, (1 << lg) - 1, l[j], r[j], 0);
			add(0, dp[i]);
			dp[i] = vv;
		}
	}
	return dp[n - 1];
}

int slowsolve() {
	vector<vector<int> > dp(n + 1, vector<int>(k + 1));
    dp[0][0] = 1;
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < k; j++) {
            int cur = 0;
            for (int o = i; o < n; o++) {
                cur ^= a[o];
                if (l[j] <= cur && cur <= r[j]) {
                    dp[o + 1][j + 1] = (dp[o + 1][j + 1] + dp[i][j]) % mod;
                }
            }
        }
    }
    return dp[n][k];
}

void gen() {
	n = rand() % 100 + 1;
	k = rand() % ((int)sqrt(n + 0.0)) + 1;
	a.resize(n);
	l.resize(k);
	r.resize(k);
	for (int i = 0; i < n; i++) {
		a[i] = rand() & 127;
	}
	for (int i = 0; i < k; i++) {
		l[i] = rand() & 127;
		r[i] = rand() & 127;
		if (l[i] > r[i]) swap(l[i], r[i]);
	}
}

void stress(bool f) {
	if (!f) return;
	for (int it = 0;; it++) {
		cout << it << endl;
		gen();
		int ans1 = fastsolve();
		int ans2 = slowsolve();
		if (ans1 != ans2) {
			cout << "fail" << endl;
			exit(0);
		}
	}
}

int main() {
	stress(1);
	freopen("xor.in", "r", stdin);
	freopen("xor.out", "w", stdout);

	scanf("%d%d", &n, &k);
	a.resize(n);
	l.resize(k);
	r.resize(k);
	for (int i = 0; i < n; i++) scanf("%d", &a[i]);
	for (int i = 0; i < k; i++) scanf("%d%d", &l[i], &r[i]);

	cout << fastsolve() << endl;

	return 0;
}