#include <cstdio>
#include <iostream>
#include <vector>
#include <cstdlib>
#include <cassert>
using namespace std;
#define NAME "xor"

struct Bor {
    static const int MOD = 1e9 + 7;
    static const int H = 30;
    struct Node;

    static void safeAdd(int& x, int y) {
        x += y;
        if (x >= MOD) x -= MOD;
    }

    struct Node {
        vector <int*> dps;
        Node* t[2];
        int add;
        Node() {
            add = 0;
            t[0] = t[1] = NULL;
        }

        void push() {
            if (add == 0)
                return;
            Node *lef = t[0];
            Node *rig = t[1];
            if (lef == NULL && rig == NULL) {
                for (int i = 0; i < dps.size(); ++i)
                    safeAdd(*dps[i], add);
            } else {
                if (lef != NULL)
                    safeAdd(lef->add, add);
                if (rig != NULL)
                    safeAdd(rig->add, add);
            }
            add = 0;
        }
    };

    int* dp;
    Node *root;
    Bor() {
        root = NULL;
        dp = NULL;
    }

    Bor(int n) {
        dp = (int*)malloc(sizeof(int) * n);
        for (int i = 0; i < n; ++i) dp[i] = 0;
        root = new Node();
    }

    bool getBit(int x, int i) {
        return (x>>i)&1;
    }

    void insert(int x, int id) {
        Node *v = root;
        for (int i = H - 1; i >= 0; --i) {
            v->push();
            bool bit = getBit(x, i);
            if (v->t[bit] == NULL)
                v->t[bit] = new Node();
            v = v->t[bit];
        }
        v->push();
        v->dps.push_back(&dp[id]);
    }

    int getDp(int id, int si) {
        Node *v = root;
        for (int i = H - 1; i >= 0; --i) {
            v->push();
            bool bit = getBit(si, i);
            assert(v->t[bit] != NULL);
            v = v->t[bit];
        }
        v->push();
        return dp[id];
    }

    void addLR(int l, int r, int si, int value) {
        Node *v = root;
        int i = H - 1;
        for (; i >= 0; --i) {
            v->push();
            bool bitl = getBit(l, i);
            bool bitr = getBit(r, i);
            bool bitsi = getBit(si, i);
            if (bitl == bitr) {
                if (v->t[bitl ^ bitsi] == NULL)
                    return;
                v = v->t[bitl ^ bitsi];
            } else
                break;
        }
        if (i == -1) {
            safeAdd(v->add, value);
            return;
        }
        bool bitsi = getBit(si, i);
        if (v->t[bitsi] != NULL)
            pushNext(v->t[bitsi], l, si, value, i - 1, 1);
        if (v->t[!bitsi] != NULL)
            pushNext(v->t[!bitsi], r, si, value, i - 1, 0);
    }

    void pushNext(Node *v, int bound, int si, int addVal, int i, bool criticalBit) {
        assert(v != NULL);
        for (;; --i) {
            v->push();
            if (i == -1)
                break;
            bool bitSi = getBit(si, i);
            bool boundBit = getBit(bound, i);
            if (boundBit != criticalBit && v->t[criticalBit ^ bitSi] != NULL)
                safeAdd(v->t[criticalBit ^ bitSi]->add, addVal);
            if (v->t[bitSi ^ boundBit] == NULL)
                break;
            v = v->t[bitSi ^ boundBit];
        }
        if (v != NULL) {
            if (i == -1)
                safeAdd(v->add, addVal);
            v->push();
        }
    }
};

const int N = 1e5 + 10;
int n, k, s[N];
int l[N], r[N];
Bor *bors;

int main() {
    freopen(NAME".in", "r", stdin);
    freopen(NAME".out", "w", stdout);
    cin >> n >> k;
    for (int i = 0; i < n; ++i) {
        int x;
        scanf("%d", &x);
        if (i == 0) s[i] = x;
        else s[i] = s[i - 1] ^ x;
    }
    for (int i = 0; i < k; ++i)
        scanf("%d %d", &l[i], &r[i]);
    bors = new Bor[k + 1];
    for (int i = 0; i <= k; ++i)
        bors[i] = Bor(n);

    for (int i = 0; i < n; ++i)
        bors[1].insert(s[i], i);
    bors[1].addLR(l[0], r[0], 0, 1);

    for (int i = 1; i < k; ++i)
        for (int j = n - 1; j >= 0; --j) {
            int dpj = bors[i].getDp(j, s[j]);
            bors[i + 1].addLR(l[i], r[i], s[j], dpj);
            bors[i + 1].insert(s[j], j);
        }
    cout << bors[k].getDp(n - 1, s[n - 1]);
    return 0;
}
